﻿using RocketAd;
using RocketGM;
using UnityEngine;

#if ROC_GM_APPSFLYER
using RocketGMAppsFlyer;
#endif
#if ROC_FACEBOOK
using RocketFacebook;
#endif
#if ROC_FIREBASE
using Firebase;
#endif

namespace RocketRevenueTracker
{
    public class RocketRevenueTracker
    {
        private bool _isSendingInProgress = false;
        
        private readonly float _revenueThreshold = 0.10f;
        private readonly string _eventSuffix = "_10";
        
        private readonly string _eventPrefix = "revenue_reached";

        private float RevenueAmount
        {
            get => PlayerPrefs.GetFloat($"RocGMRevenueTrackerAmount{_eventSuffix}", 0f);
            set => PlayerPrefs.SetFloat($"RocGMRevenueTrackerAmount{_eventSuffix}", value);
        }

        public RocketRevenueTracker(float threshold, string suffix)
        {
            _revenueThreshold = threshold;
            _eventSuffix = suffix;

            RocGm.DoAfterInitialized( () =>
            {
                TrySendRevenue(_revenueThreshold);
            });
            
            RocAd.ImpressionTracked += OnImpressionTracked;
        }

        ~RocketRevenueTracker()
        {
            RocAd.ImpressionTracked -= OnImpressionTracked;
        }        

        private void OnImpressionTracked(RocImpressionData rocImpressionData)
        {
            RevenueAmount += (float)rocImpressionData.Revenue;
            RocGm.DoAfterInitialized( () =>
            {
                TrySendRevenue(_revenueThreshold);
            });
        }

        private void TrySendRevenue(float revenueAmount)
        {
            if (RevenueAmount < _revenueThreshold || _isSendingInProgress)
                return;
            
            _isSendingInProgress = true;
            
            var eventName = $"{_eventPrefix}{_eventSuffix}";
            
#if ROC_GM_APPSFLYER
            if (RocGMAppsFlyerController.Instance != null) RocGMAppsFlyerController.Instance.Report(eventName);
#endif
#if ROC_FACEBOOK
            RocFacebookController.SendCustomEvent(eventName);
#endif
#if ROC_FIREBASE
            RocFirebaseController.SendCustomEvent(eventName);
#endif
            
            RevenueAmount -= revenueAmount;

            _isSendingInProgress = false;
        }
    }
}